/*********************************************************/
/***              STRYK System Improvement             ***/
/***    Performance Optimization & Troubleshooting     ***/
/***  (c) 2008, STRYK System Improvement, Jrg Stryk   ***/
/***                   www.Stryk.info                  ***/
/*********************************************************/

DROP PROCEDURE [dbo].[ssi_blockdetection] 
GO

CREATE PROCEDURE [dbo].[ssi_blockdetection] AS
IF EXISTS (SELECT * FROM sys.dm_exec_requests WHERE [blocking_session_id] <> 0) BEGIN
    SET STATISTICS IO OFF
    /* Saving initial data */
	INSERT INTO [dbo].[ssi_BlockLog]
	(      
		   [timestamp]
		  ,[waitresource]
		  ,[table_name]
		  ,[index_name]
		  ,[waittime]
		  ,[lastwaittype]
		  ,[spid]
		  ,[loginame]
		  ,[hostname]
		  ,[cmd]          
		  ,[status]
		  ,[cpu]
		  ,[blocked by]
		  ,[loginame 2]
		  ,[hostname 2]  
		  ,[cmd 2]   
		  ,[status 2]
		  ,[cpu 2]
		  ,[db]
	)  
		SELECT [timestamp] = GETDATE(),
		   s1.[wait_resource],
		   null,
		   null  ,
		   s1.[wait_time],
		   s1.[last_wait_type],
		   session1.[session_id],
		   session1.[login_name],
		   session1.[host_name],
		   s1.[command],
		   session1.[status],
		   session1.[cpu_time],
		   [blocked by] = s1.[blocking_session_id],
		   [loginame 2] = session2.[login_name],
		   [hostname 2] = session2.[host_name],
		   [cmd 2] = s2.[command],
		   [status 2] = session2.[status],
		   [cpu 2] = session2.[cpu_time],
		   [db] = DB_NAME(s1.[database_id])
		   FROM sys.dm_exec_requests (NOLOCK) s1 
		   LEFT JOIN sys.dm_exec_requests (NOLOCK) s2 ON s2.[session_id] = s1.[blocking_session_id]
		   LEFT JOIN sys.dm_exec_sessions (NOLOCK) session1 ON session1.[session_id] = s1.[session_id]
		   LEFT JOIN sys.dm_exec_sessions (NOLOCK) session2 ON session2.[session_id] = s1.[blocking_session_id]
		   WHERE s1.[blocking_session_id] <> 0

    /* Updating data, Parsing "wait_resource */
    DECLARE @waitresource VARCHAR(128), @dbid INT, @objid BIGINT, @indid INT, @partid BIGINT, @idxname VARCHAR(128)
    DECLARE @i_db INT, @i_tab INT, @i_idx INT, @i_part INT
    DECLARE waitresource_cur CURSOR FOR SELECT LTRIM(RTRIM([waitresource])) FROM [dbo].[ssi_BlockLog] WHERE [table_name] IS NULL FOR UPDATE
    OPEN waitresource_cur
    FETCH NEXT FROM waitresource_cur INTO @waitresource
    WHILE @@FETCH_STATUS = 0 BEGIN
      SET @i_db = 0
      SET @i_tab = 0
      SET @i_idx = 0
      SET @i_part = 0
      SET @i_db = CHARINDEX(':', @waitresource)
      SET @i_tab = CHARINDEX(':', @waitresource, @i_db + 1)
      IF @i_tab <> 0
        SET @i_idx = CHARINDEX(':', @waitresource, @i_tab + 1)
      IF @i_idx = 0 AND @i_tab <> 0 
        SET @i_part = CHARINDEX('(', @waitresource, @i_tab + 1)
      IF @i_db <> 0 AND @i_tab <> 0
        SET @dbid = LTRIM(SUBSTRING(@waitresource, @i_db + 1, @i_tab - @i_db - 1))      
      IF @i_tab <> 0 AND @i_idx <> 0 BEGIN
        SET @objid = LTRIM(SUBSTRING(@waitresource, @i_tab + 1, @i_idx - @i_tab - 1))
        UPDATE [dbo].[ssi_BlockLog] SET [table_name] = OBJECT_NAME(@objid) WHERE CURRENT OF waitresource_cur
      END
      IF @i_idx <> 0 BEGIN
        SET @indid = LTRIM(SUBSTRING(@waitresource, @i_idx + 1, LEN(@waitresource) - @i_idx))
        SELECT @idxname = "name" FROM sys.indexes (NOLOCK) WHERE "object_id" = @objid AND "index_id" = @indid
        UPDATE [dbo].[ssi_BlockLog] SET [index_name] = @idxname WHERE CURRENT OF waitresource_cur
      END
      IF @i_tab <> 0 AND @i_part <> 0 BEGIN
        SET @partid = LTRIM(SUBSTRING(@waitresource, @i_tab + 1, @i_part - @i_tab - 1))
        SELECT @objid = "object_id", @indid = "index_id" FROM sys.partitions (NOLOCK) WHERE "partition_id" = @partid
        UPDATE [dbo].[ssi_BlockLog] SET [table_name] = OBJECT_NAME(@objid) WHERE CURRENT OF waitresource_cur
        SELECT @idxname = "name" FROM sys.indexes (NOLOCK) WHERE "object_id" = @objid AND "index_id" = @indid
        UPDATE [dbo].[ssi_BlockLog] SET [index_name] = @idxname WHERE CURRENT OF waitresource_cur
      END
      FETCH NEXT FROM waitresource_cur INTO @waitresource
    END
    CLOSE waitresource_cur
    DEALLOCATE waitresource_cur
END